/// <reference types="react" />
import { TransitionPlainStyle, Style, PlainStyle } from "react-motion";
import Phases from "./Phases";
export declare type SceneBundleThunk = () => Promise<any>;
export declare type StyleCalculator = (style: PlainStyle, phase: Phases) => Style;
export declare type StyleCalculators = {
    container: StyleCalculator;
    loadingPlay: StyleCalculator;
    scenePlay: StyleCalculator;
};
export declare type NextPhaseChecker = (style: PlainStyle, isSceneReady: boolean) => boolean | null | undefined;
export declare type NextPhaseCheckers = {
    container: NextPhaseChecker;
    loadingPlay: NextPhaseChecker;
    scenePlay: NextPhaseChecker;
};
export declare type StyleKeys = "container" | "loadingPlay" | "scenePlay";
export interface InitMotionStyle extends TransitionPlainStyle {
    key: StyleKeys;
}
export declare type NumberToStyle = (style: PlainStyle, phase: Phases, isSceneReady: boolean) => {
    [key: string]: string;
};
export declare type NumberToStyles = {
    container: NumberToStyle;
    loadingPlay: NumberToStyle;
    scenePlay: NumberToStyle;
};
export declare type Props = {
    loadingPlay: React.ReactNode;
    sceneBundleThunk: SceneBundleThunk;
    children: (bundle: any) => React.ReactNode;
    initStyles: InitMotionStyle[];
    styleCalculators: StyleCalculators;
    nextPhaseCheckers: NextPhaseCheckers;
    numberToStyles: NumberToStyles;
};
export declare type Actions = {
    setState: (state: State) => void;
    nextPhase: (phase: Phases) => void;
    startLeaving: () => void;
    loadSceneBundle: (sceneBundleThunk: SceneBundleThunk) => void;
};
export declare type ConnectedProps = {
    actions: Actions;
} & Props & State;
export declare type State = {
    isSceneReady: boolean;
    phase: Phases;
    bundle: any | null;
};
export declare type ExtendedStyleKeys = StyleKeys | "nextPhase";
export interface ExtendedMotionStyle extends TransitionPlainStyle {
    key: StyleKeys | "nextPhase";
}
export interface ExtendedPlainMotionStyle extends TransitionPlainStyle {
    key: StyleKeys | "nextPhase";
}
export declare type CombinedStyleCalculator = (prevStyles: ExtendedPlainMotionStyle[]) => ExtendedMotionStyle[];
