/// <reference types="react" />
import { ReactNode } from "react";
import { TransitionPlainStyle, Style, PlainStyle } from "react-motion";
import Phases from "./Phases";
import PlayStrategies from "./PlayStrategies";
export declare type StyleCalculator = (style: PlainStyle, phase: Phases) => Style;
export declare type StyleCalculators = {
    container: StyleCalculator;
    oldPlay: StyleCalculator;
    newPlay: StyleCalculator;
};
export declare type StyleKeys = "container" | "oldPlay" | "newPlay";
export interface InitMotionStyle extends TransitionPlainStyle {
    key: StyleKeys;
}
export declare type NumberToStyle = (style: PlainStyle, phase: Phases) => {
    [key: string]: string;
};
export declare type NumberToStyles = {
    container: NumberToStyle;
    oldPlay: NumberToStyle;
    newPlay: NumberToStyle;
};
export declare type NextPhaseChecker = (style: PlainStyle) => boolean | null | undefined;
export declare type NextPhaseCheckers = {
    container: NextPhaseChecker;
    oldPlay: NextPhaseChecker;
    newPlay: NextPhaseChecker;
};
export declare type ExtendedStyleKeys = StyleKeys | "nextPhase";
export interface ExtendedMotionStyle extends TransitionPlainStyle {
    key: StyleKeys | "nextPhase";
}
export interface ExtendedPlainMotionStyle extends TransitionPlainStyle {
    key: StyleKeys | "nextPhase";
}
export declare type CombinedStyleCalculator = (prevStyles: ExtendedPlainMotionStyle[]) => ExtendedMotionStyle[];
export declare type Props = {
    children: ReactNode;
    initStyles: InitMotionStyle[];
    styleCalculators: StyleCalculators;
    nextPhaseCheckers: NextPhaseCheckers;
    numberToStyles: NumberToStyles;
};
export declare type Actions = {
    setState: (state: State) => void;
    nextPhase: (phase: Phases, oldPlayKey: string, oldPlay: PlayEntity) => void;
    addPlay: (node: ReactNode) => void;
    removePlay: (node: ReactNode) => void;
    updatePlay: (oldNode: ReactNode, newNode: ReactNode) => void;
    playNext: (PlayStrategies?: PlayStrategies) => void;
};
export declare type ConnectedProps = {
    actions: Actions;
} & Props & State;
export declare type PlayEntity = {
    node?: ReactNode;
};
export declare type State = {
    play1: PlayEntity;
    play2: PlayEntity;
    newPlayKey: "play1" | "play2";
    playlist: Array<PlayEntity>;
    autoClearPlay: PlayEntity | null;
    phase: Phases;
};
