Object.defineProperty(exports, "__esModule", { value: true });
var state_1 = require("./state");
var ActionTypes_1 = require("./ActionTypes");
var Phases_1 = require("./Phases");
var PlayStrategies_1 = require("./PlayStrategies");
function nextPhaseState(state, action) {
    var phase = action.phase, oldPlayKey = action.oldPlayKey, oldPlay = action.oldPlay;
    if (state.phase !== phase || state[oldPlayKey] !== oldPlay)
        return state;
    switch (state.phase) {
        case Phases_1.default.ENTERING:
            var newPlayKey = void 0;
            if (state.newPlayKey === "play2") {
                newPlayKey = "play1";
            }
            else {
                newPlayKey = "play2";
            }
            return Object.assign({}, state, (_a = {
                    phase: Phases_1.default.IN
                },
                _a[newPlayKey] = {},
                _a.newPlayKey = newPlayKey,
                _a));
        case Phases_1.default.LEAVING:
            return Object.assign({}, state, { phase: Phases_1.default.OUT });
        case Phases_1.default.OUT:
            return Object.assign({}, state, { phase: Phases_1.default.ENTERING });
        default:
            return state;
    }
    var _a;
}
function playRemoveState(state, action) {
    var oldPlayKey = state.newPlayKey === "play1" ? "play2" : "play1";
    if (state.phase == Phases_1.default.IN &&
        state[oldPlayKey].node === action.node) {
        return Object.assign({}, state, { phase: Phases_1.default.LEAVING });
    }
    else if (state[state.newPlayKey].node === action.node) {
        return Object.assign({}, state, { autoClearPlay: action.node });
    }
    else {
        return Object.assign({}, state, {
            playlist: state.playlist.filter(function (entity) { return entity.node !== action.node; })
        });
    }
}
function playUpdateState(state, action) {
    var oldPlayKey = state.newPlayKey === "play1" ? "play2" : "play1";
    var oldNode = action.oldNode, newNode = action.newNode;
    if (state[oldPlayKey].node === oldNode) {
        return Object.assign({}, state, (_a = {}, _a[oldPlayKey] = { node: newNode }, _a));
    }
    else if (state[state.newPlayKey].node === oldNode) {
        return Object.assign({}, state, (_b = {},
            _b[state.newPlayKey] = { node: newNode },
            _b));
    }
    else {
        var index = state.playlist.findIndex(function (entity) { return entity.node === oldNode; });
        if (index > -1) {
            return Object.assign({}, state, {
                playlist: state.playlist
                    .slice(0, index)
                    .concat({ node: newNode })
                    .concat(state.playlist.slice(index + 1))
            });
        }
        return state;
    }
    var _a, _b;
}
function reducer(state, action) {
    if (state === void 0) { state = state_1.default; }
    switch (action.type) {
        case ActionTypes_1.default.ARENA_SWITCH_ANIMATION_NEXTPHRASE:
            return nextPhaseState(state, action);
        case ActionTypes_1.default.ARENA_SWITCH_ANIMATION_PLAY_NEXT:
            if (state.playlist.length === 0 && state.autoClearPlay == null)
                return state;
            var newState = Object.assign({}, state);
            if (state.phase === Phases_1.default.IN) {
                newState.phase = Phases_1.default.LEAVING;
            }
            else if (state.phase === Phases_1.default.OUT && state.autoClearPlay != null) {
                newState.autoClearPlay = null;
            }
            else {
                return state;
            }
            if (state.playlist.length > 0) {
                if (state.play1.node == null && state.play2.node == null) {
                    newState.phase = Phases_1.default.OUT;
                }
                switch (action.PlayStrategies) {
                    case PlayStrategies_1.default.PLAT_NEXT:
                        newState[state.newPlayKey] = state.playlist[0];
                        newState.playlist = state.playlist.slice(1);
                    default:
                        newState[state.newPlayKey] =
                            state.playlist[state.playlist.length - 1];
                        newState.playlist = [];
                }
            }
            return newState;
        case ActionTypes_1.default.ARENA_SWITCH_ANIMATION_PLAY_ADD:
            return Object.assign({}, state, {
                playlist: state.playlist.concat(action.entity)
            });
        case ActionTypes_1.default.ARENA_SWITCH_ANIMATION_PLAY_REMOVE:
            return playRemoveState(state, action);
        case ActionTypes_1.default.ARENA_SWITCH_ANIMATION_PLAY_UPDATE:
            return playUpdateState(state, action);
        default:
            return state;
    }
}
exports.default = reducer;
